@@ -352,7 +352,7 @@ window_destroy (GtkWidget* widget, gpointer data)
 }
 static void show_candwords()
 {
-  gchar tmp[MAX_USER_INPUT + 1];
+  gchar tmp[MAX_WORDS_USER_INPUT * UTF8_MAX_LENGTH + 1];
   gchar *wd = tmp;
   tmp[0] = '\0';
   if(iPYInsertPoint){
@@ -505,6 +505,10 @@ gboolean fake_key_press(guint keyval, guint state, gchar **str, gboolean *ret)
   }
   if(!chinese_mode || hildon_im_ui_get_shift_locked(client) || hildon_im_ui_get_level_locked(client))
     return TRUE;
+  if(iPYInsertPoint >= MAX_USER_INPUT){
+    *ret = TRUE;
+    return TRUE;
+  }
   if(((keyval >= 'a' && keyval <= 'z') || ((keyval == ',' || keyval == '.') && iPYInsertPoint)) && state == 0){
       DoPYInput(keyval);
       show_candwords();
@@ -589,13 +593,14 @@ gboolean fake_key_press(guint keyval, guint state, gchar **str, gboolean *ret)
     *ret = TRUE;
   }else if(iPYInsertPoint && (keyval == GDK_Return   ||
                               keyval == GDK_KP_Enter ||
+                              (state == 128 && keyval == '@') ||
                               keyval == GDK_ISO_Enter)){
     commit_text(client,strCodeInput,str);
     ResetPYStatus();
     show_candwords();
     *ret = TRUE;
   }else if(iPYInsertPoint && ((state == 0 && (keyval == GDK_Control_L || keyval == GDK_Shift_L)) ||
-                             (state == 128 && (keyval != GDK_Up || keyval != GDK_Down)))){
+                             (state == 128 && (keyval != GDK_Up || keyval != GDK_Down || keyval != '@')))){
       ResetPYStatus();
       show_candwords();
       *ret = FALSE;
